from lcrequests import Request
from lcrequests.exceptions import LCHTTPError
from typing import TypedDict


class Impersonation(TypedDict):
    """
    Odoo will send a response back configuration data
    for the Impersonated and Impersonator systems.

    {
        'impersonated': {
            'cid': '6bb3a862-892c-6424-8b60-12a79900e8f7',
            'is_active': True,
            'rebranded': False,
            'theme': 'default',
            'name': 'thinkwell',
            'adminurl': 'thinkwell.poweredbyclear.com',
            'device_dns_name': 'thinkwell.orbitmobile.network',
            'external_port': 443,
            'maincomp': '1e770796-6ca2-6fb8-9123-f80f41f8bd97',
            'ddns_url': 'https://ipv4.cloudns.net/api/dynamicURL/?q=Mjc0ODY5OToyMDMxMTY5MDc6ZmJjNjEwYjMwNWFkNzBiZDI3ODliMTkzMDM5YzYzOWYyZjgwY2E0ZjNjZDk4OGFlNTRhOGQ1YTY2Y2U5OWQ2NQ',
            'sync_publisher_data': {
                'url': 'willow.compassfoundation.io',
                'port': 1525,
                'mode': 'full'
            },
            'auth_token': 'f9a9aa2838bf64d158cda1187f85d65955a7846c'
        },
        'impersonator': {
            'cid': '4fcb3c9b-c3bd-6a61-a7d1-862a715b7f67',
            'is_active': True,
            'rebranded': False,
            'theme': 'default',
            'name': 'marvinmartin',
            'adminurl': 'marvinmartin.poweredbyclear.com',
            'device_dns_name': 'marvinmartin.orbitmobile.network',
            'external_port': 0,
            'maincomp': 'a4a53a17-470b-6e89-b211-a1d8b583abd6',
            'auth_token': '078be7c375591f421476701ef35ff864a42c3a53'
        }
    }
    """
    impersonated: dict
    impersonator: dict


def odoo_impersonation_data(cid: str, otp: str) -> Impersonation:
    """
    Attempt to retrieve system data from Odoo that to enable provisioning
    this system as an impersonator of the failed Appliance.
    """
    headers = {
        'content-type': 'application/x-www-form-urlencoded',
        'charset': 'utf-8',
    }
    url = f'https://erp.compassfoundation.io/appliance/impersonate/{cid}/'
    try:
        req = Request(url, headers=headers)
        resp = req.post(body={'otp': otp})
        if not resp.ok:
            raise SystemExit(f'\nFailed to retrieve data from Odoo: {resp.json()}\n')
    except LCHTTPError as e:
        msg = e.response.json()
        raise SystemExit(f'Unable to retrieve impersonation data - Msg: \n\n{msg}\n')
    except Exception as e:
        raise SystemExit(f'Unable to contact Odoo - Error: \n\n{e}\n')

    odoo_data: Impersonation = resp.json()
    return odoo_data


__all__ = [
    'odoo_impersonation_data',
]
